/* Copyright (c) 2013 Nordic Semiconductor. All Rights Reserved.
 *
 * Use of this source code is governed by a BSD-style license that can be
 * found in the license.txt file.
 */

#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "app_error.h"
#include "nrf_gpio.h"
#include "nrf51_bitfields.h"
#include "ble.h"
#include "ble_hci.h"
#include "ble_srv_common.h"
#include "ble_advdata.h"
#include "ble_conn_params.h"
#include "boards.h"
#include "app_scheduler.h"
#include "softdevice_handler.h"
#include "app_timer.h"
#include "ble_error_log.h"
#include "ble_l2cap.h"
#include "app_gpiote.h"
#include "ble_debug_assert_handler.h"
#include "pstorage.h"
#include "ble_lbs.h"
#include "nrf_delay.h"
#include "simple-uart.h"

#define APP_TIMER_PRESCALER             2047						                    /**< Value of the RTC1 PRESCALER register. */
#define APP_TIMER_MAX_TIMERS            3                                           /**< Maximum number of simultaneously created timers. */
#define APP_TIMER_OP_QUEUE_SIZE         4                                           /**< Size of timer operation queues. */

#define CID_USED						BLE_L2CAP_CID_DYN_BASE + 1

#define DEAD_BEEF                       0xDEADBEEF                                  /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */

static app_timer_id_t 					alarm_state_timer;
static uint16_t                         m_conn_handle = 0x1234;    /**< Handle of the current connection. */

#define SCHED_MAX_EVENT_DATA_SIZE       sizeof(app_timer_event_t)                   /**< Maximum size of scheduler events. Note that scheduler BLE stack events do not contain any data, as the events are being pulled from the stack in the event handler. */
#define SCHED_QUEUE_SIZE                10                                          /**< Maximum number of events in the scheduler queue. */

// Persistent storage system event handler
void pstorage_sys_event_handler (uint32_t p_evt);
static void alarm_timer_start(uint32_t counts);

/**@brief Function for error handling, which is called when an error has occurred.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyze
 *          how your product is supposed to react in case of error.
 *
 * @param[in] error_code  Error code supplied to the handler.
 * @param[in] line_num    Line number where the handler is called.
 * @param[in] p_file_name Pointer to the file name.
 */
void app_error_handler(uint32_t error_code, uint32_t line_num, const uint8_t * p_file_name)
{
	// This call can be used for debug purposes during application development.
	// @note CAUTION: Activating this code will write the stack to flash on an error.
	//                This function should NOT be used in a final product.
	//                It is intended STRICTLY for development/debugging purposes.
	//                The flash write will happen EVEN if the radio is active, thus interrupting
	//                any communication.
	//                Use with care. Un-comment the line below to use.
	//ble_debug_assert_handler(error_code, line_num, p_file_name);

	printf("Error of %X at %d in %s\n",error_code,line_num,p_file_name);

	for(;;);

	// On assert, the system can only recover with a reset.
	//NVIC_SystemReset();
}


/**@brief Callback function for asserts in the SoftDevice.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *BLE_L2CAP_CID_DYN_BASE + 1
 * @warning This handler is an example only and does not fit a final product. You need to analyze
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in]   line_num   Line number of the failing ASSERT call.
 * @param[in]   file_name  File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
	app_error_handler(DEAD_BEEF, line_num, p_file_name);
}
#define APP_GPIOTE_MAX_USERS            1                                           /**< Maximum number of users of the GPIOTE handler. */


/**@brief Function for the LEDs initialization.
 *
 * @details Initializes all LEDs used by the application.
 */
static void leds_init(void)
{
	nrf_gpio_cfg_output(LED_RGB_BLUE);
	nrf_gpio_cfg_output(LED_RGB_RED);
	nrf_gpio_cfg_output(LED_RGB_GREEN);
	nrf_gpio_pin_set(LED_RGB_RED);
	nrf_gpio_pin_set(LED_RGB_BLUE);
	nrf_gpio_pin_set(LED_RGB_GREEN);
}

static void alarm_state_timer_handler(void * p_context){
	uint32_t err_code;
	ble_l2cap_header_t tx_head;
	uint8_t tx_buffer_count;

	uint8_t tx_data[] = "Howdy!";

	tx_head.cid = CID_USED;
	tx_head.len = sizeof(tx_data);

	nrf_gpio_pin_toggle(LED_RGB_RED);

	err_code = sd_ble_tx_buffer_count_get(&tx_buffer_count);
	APP_ERROR_CHECK(err_code);
	printf("Count:%d\n",tx_buffer_count);

	err_code = sd_ble_l2cap_tx (m_conn_handle, &tx_head, tx_data);
	APP_ERROR_CHECK(err_code);
}

static void uart_event_handler(uint8_t rx_data)
{
}

/**@brief Function for the UART initialization.
 *
 * @details Initializes the UART module.
 */
static void uart_init()
{
	simple_uart_config(RTS_PIN_NUMBER, TX_PIN_NUMBER, CTS_PIN_NUMBER, RX_PIN_NUMBER,
			UART_BAUDRATE_BAUDRATE_Baud38400, APP_IRQ_PRIORITY_LOW, HWFC);
	uart_handler_init(uart_event_handler);
}

/**@brief Function for the Timer initialization.
 *
 * @details Initializes the timer module.
 */
static void timers_init(void)
{
	// Initialize timer module, making it use the scheduler
	APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_MAX_TIMERS, APP_TIMER_OP_QUEUE_SIZE, true);

	uint32_t err_code = app_timer_create(&alarm_state_timer, APP_TIMER_MODE_REPEATED, alarm_state_timer_handler);
	APP_ERROR_CHECK(err_code);
}


/**@brief Function for starting Alarm timer.
 */
static void alarm_timer_start(uint32_t counts)
{
	uint32_t err_code = app_timer_start(alarm_state_timer, counts, NULL);
	APP_ERROR_CHECK(err_code);
}

/**@brief Function for handling the Application's BLE Stack events.
 *
 * @param[in]   p_ble_evt   Bluetooth stack event.
 */
static void on_ble_evt(ble_evt_t * p_ble_evt)
{
//	uint32_t                         err_code;

	switch (p_ble_evt->header.evt_id)
	{
	case BLE_L2CAP_EVT_RX:
		simple_uart_putstring("L2capRx\n");
		break;

	case BLE_EVT_TX_COMPLETE:
		simple_uart_putstring("TXComplete\n");
		break;

	default:
		// No implementation needed.
		break;
	}
}


/**@brief Function for dispatching a BLE stack event to all modules with a BLE stack event handler.
 *
 * @details This function is called from the scheduler in the main loop after a BLE stack
 *          event has been received.
 *
 * @param[in]   p_ble_evt   Bluetooth stack event.
 */
static void ble_evt_dispatch(ble_evt_t * p_ble_evt)
{
	on_ble_evt(p_ble_evt);
}


/**@brief Function for dispatching a system event to interested modules.
 *
 * @details This function is called from the System event interrupt handler after a system
 *          event has been received.
 *
 * @param[in]   sys_evt   System stack event.
 */
static void sys_evt_dispatch(uint32_t sys_evt)
{
	pstorage_sys_event_handler(sys_evt);
}

static void l2cap_init(void){
	uint32_t err_code = sd_ble_l2cap_cid_register(CID_USED);
	APP_ERROR_CHECK(err_code);
}

/**@brief Function for initializing the BLE stack.
 *
 * @details Initializes the SoftDevice and the BLE event interrupt.
 */
static void ble_stack_init(void)
{
	uint32_t err_code;

	// Initialize the SoftDevice handler module.
	SOFTDEVICE_HANDLER_INIT(NRF_CLOCK_LFCLKSRC_XTAL_20_PPM, false);
	// Register with the SoftDevice handler module for BLE events.
	err_code = softdevice_ble_evt_handler_set(ble_evt_dispatch);
	APP_ERROR_CHECK(err_code);
	// Register with the SoftDevice handler module for BLE events.
	err_code = softdevice_sys_evt_handler_set(sys_evt_dispatch);
	APP_ERROR_CHECK(err_code);
}


/**@brief Function for the Event Scheduler initialization.
 */
static void scheduler_init(void)
{
	APP_SCHED_INIT(SCHED_MAX_EVENT_DATA_SIZE, SCHED_QUEUE_SIZE);
}

 /**@brief Function for the Power manager.
  */
 static void power_manage(void)
 {
	 uint32_t err_code = sd_app_evt_wait();
	 APP_ERROR_CHECK(err_code);
 }


 /**@brief Function for application main entry.
  */
 int main(void)
 {
	 // Initialize
	 leds_init();
	 timers_init();
	 uart_init();

	 ble_stack_init();
	 scheduler_init();
	 l2cap_init();

	 // Start execution
	 alarm_timer_start(APP_TIMER_CLOCK_FREQ/(APP_TIMER_PRESCALER+1));

	 // Enter main loop
	 for (;;)
	 {
		 app_sched_execute();
		 power_manage();
	 }
 }

 /**
  * @}
  */
